"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_codegen_visitor_plugin_common_1 = require("graphql-codegen-visitor-plugin-common");
const graphql_1 = require("graphql");
class TypeScriptOperationVariablesToObject extends graphql_codegen_visitor_plugin_common_1.OperationVariablesToObject {
    constructor(_scalars, _convertName, _avoidOptionals, _immutableTypes) {
        super(_scalars, _convertName);
        this._avoidOptionals = _avoidOptionals;
        this._immutableTypes = _immutableTypes;
    }
    clearOptional(str) {
        if (str.startsWith('Maybe')) {
            return str.replace(/^Maybe<(.*?)>$/i, '$1');
        }
        return str;
    }
    wrapAstTypeWithModifiers(baseType, typeNode) {
        if (typeNode.kind === graphql_1.Kind.NON_NULL_TYPE) {
            const type = this.wrapAstTypeWithModifiers(baseType, typeNode.type);
            return this.clearOptional(type);
        }
        else if (typeNode.kind === graphql_1.Kind.LIST_TYPE) {
            const innerType = this.wrapAstTypeWithModifiers(baseType, typeNode.type);
            return `Maybe<${this._immutableTypes ? 'ReadonlyArray' : 'Array'}<${innerType}>>`;
        }
        else {
            return `Maybe<${baseType}>`;
        }
    }
    formatFieldString(fieldName, isNonNullType, hasDefaultValue) {
        if (hasDefaultValue || isNonNullType || this._avoidOptionals) {
            return fieldName;
        }
        else {
            return `${fieldName}?`;
        }
    }
    formatTypeString(fieldType, isNonNullType, hasDefaultValue) {
        if (hasDefaultValue) {
            return this.clearOptional(fieldType);
        }
        return fieldType;
    }
}
exports.TypeScriptOperationVariablesToObject = TypeScriptOperationVariablesToObject;
//# sourceMappingURL=typescript-variables-to-object.js.map